use serde::{Deserialize, Serialize};
use tracing::{info, instrument};

use crate::db::concour::Concour;

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum AddKeywordConcourError {
    DoesntExist,
    FindError(String),
    UnknownError(String),
}

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum KeyWordSource {
    /// Fetch content from url and add to keywords, if delimiter is provided split the content with the delimiter either use \n
    Url(String, Option<String>),
    /// Add text to keywords, if delimiter is provided split the content with the delimiter either use \n
    Text(String, Option<String>),
}

/// Add keyword to concour
#[instrument(level = "info")]
pub async fn add_keyword_concour(
    server_id: u64,
    channel_id: u64,
    keywordlist: KeyWordSource,
) -> Result<Option<Concour>, AddKeywordConcourError> {
    let concour = match Concour::find_by_server_id_channel_id(&server_id, &channel_id).await {
        Ok(list_concour) => list_concour,
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error finding concour");
            return Err(AddKeywordConcourError::UnknownError(
                "Error finding concour".to_string(),
            ));
        }
    };

    if concour.is_none() {
        info!("Concour doesn't exist");
        return Err(AddKeywordConcourError::DoesntExist);
    }

    let mut concour = concour.unwrap();

    match keywordlist {
        KeyWordSource::Url(url, delimiter) => {
            // Check if url is valid
            // if valid fetch the content and add to keywords
            match reqwest::get(url.clone()).await {
                Ok(res) => {
                    let text = res.text().await.unwrap();
                    let split_delimiter = delimiter.unwrap_or("\n".to_string());
                    text.split(&split_delimiter).for_each(|x| {
                        concour.keywords.push(x.to_string().replace('\r', ""));
                    });
                }
                Err(err) => {
                    tracing::error!(error = err.to_string(), "Error fetching url");
                    return Err(AddKeywordConcourError::UnknownError(
                        "Error fetching url".to_string(),
                    ));
                }
            }
        }
        KeyWordSource::Text(text, delimiter) => {
            let split_delimiter = delimiter.unwrap_or("\n".to_string());
            text.split(&split_delimiter).for_each(|x| {
                concour.keywords.push(x.to_string());
            });
        }
    }

    match concour.update().await {
        Ok(_) => {}
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error updating concour");
            return Err(AddKeywordConcourError::UnknownError(
                "Error updating concour".to_string(),
            ));
        }
    }
    Ok(Some(concour))
}
