use serde::{Deserialize, Serialize};
use tracing::instrument;

use crate::db::server_config::ServerConfig;

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum EnableFeatureConcourError {
    UnknownError(String),
}

#[instrument(level = "info")]
pub async fn enable_feature_concour(
    server_id: u64,
    enable: bool,
) -> Result<(), EnableFeatureConcourError> {
    let server_config =
        match crate::db::server_config::ServerConfig::find_by_server_id(&server_id).await {
            Ok(server_config) => server_config,
            Err(err) => {
                tracing::error!(error = err.to_string(), "Error finding server config");
                return Err(EnableFeatureConcourError::UnknownError(
                    "Error finding server config".to_string(),
                ));
            }
        };
    match server_config {
        None => {
            tracing::info!("Server config not found, creating it");
            let mut server_config = ServerConfig::new(server_id).unwrap();
            server_config.auto_concour = enable;
            match server_config.create().await {
                Ok(_) => Ok(()),
                Err(err) => {
                    tracing::error!(error = err.to_string(), "Error creating server config");
                    return Err(EnableFeatureConcourError::UnknownError(
                        "Error creating server config".to_string(),
                    ));
                }
            }
        }
        Some(mut server_config) => {
            if server_config.auto_concour == enable {
                return Ok(());
            } else {
                server_config.auto_concour = enable;
                match server_config.update().await {
                    Ok(_) => Ok(()),
                    Err(err) => {
                        tracing::error!(error = err.to_string(), "Error updating server config");
                        return Err(EnableFeatureConcourError::UnknownError(
                            "Error updating server config".to_string(),
                        ));
                    }
                }
            }
        }
    }
}
